/* Developed for Lenovo Educational Board Pass 2
   © Lenovo Created by Joshua Grosser
   v. 1.0 */

#include <WiFi.h>
#include <WiFiMulti.h>
#include <WiFiClientSecure.h>
#include <WebSocketsServer_Generic.h>
#include <Preferences.h>

// define websocket information
#define WS_PORT 8080
#define _WEBSOCKETS_LOGLEVEL_ 2

// create objects
Preferences preferences;
WiFiMulti WiFiMulti;
WebSocketsServer  webSocket = WebSocketsServer(WS_PORT);

// define the LED pin
#define PIN_OUT 10
// define the unit length in ms
#define UNIT_LENGTH 250
// define the button pin to input morse code
#define BUTTON 1
// define the button pin to read morse code
const int buttonTwo = 2;
// define the buton to listen
const int buttonThree = 3;
// determines whether to output to the LED
bool complete = false; 
// message in letters
String finalMsg = ""; 
// used to tell where to send to
uint8_t nums = 0;
// used for translation to and from morse code
static String codex = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890 .,?!:;()@&"; 
String ssid = "";
String password = "";

// array where the translations to morse code are stored
static String dots_and_dashes[] = {
  ".-",
  "-...",
  "-.-.",
  "-..",
  ".",
  "..-.",
  "--.",
  "....",
  "..",
  ".---",
  "-.-",
  ".-..",
  "--",
  "-.",
  "---",
  ".--.",
  "--.-",
  ".-.",
  "...",
  "-",
  "..-",
  "...-",
  ".--",
  "-..-",
  "-.--",
  "--..",
  ".----",
  "..---",
  "...--",
  "....-",
  ".....",
  "-....",
  "--...",
  "---..",
  "----.",
  "-----",
  "       ",
  ".-.-.-",
  "--..--",
  "..--..",
  "-.-...",
  "---...",
  "-.-.-.",
  "-.--.",
  "-.--.-",
  ".--.-.",
  ".-...",
};

// get Wifi info
void getWifi()
{
  preferences.begin("credentials", false);
  ssid = preferences.getString("ssid", ""); 
  password = preferences.getString("password", "");
  if (ssid == "" || password == ""){
    Serial.println("No ssid or password saved.");
  }
  else {
    Serial.println("Successfully read WiFi information.");
  }
}

// receives an event from websocket and processes whether event is text, disconnected, or connnected
void webSocketEvent(const uint8_t& num, const WStype_t& type, uint8_t * payload, const size_t& length)
{
  nums = num;
  switch (type) 
  {
    case WStype_DISCONNECTED:
    {
      Serial.printf("Disconnected!\n");
      break;
    }
    case WStype_CONNECTED:
      {
        IPAddress ip = webSocket.remoteIP(num);
        Serial.printf("Connected from %d.%d.%d.%d\n", ip[0], ip[1], ip[2], ip[3]);
        break;
      }
    case WStype_TEXT:
    {
      Serial.println();
      Serial.printf("Text Recieved: %s\n", payload);
      String str = (char*)payload;
      outputToLED(str);
      // send data to all connected clients
      // webSocket.broadcastTXT(num, messageFromServer);
      break;
    }
    default:
    {
      break;
    }  
  }
}

void setup()
{
  // set baud rate
  Serial.begin(115200);
  getWifi();
  // sets pins to appropriate mode
  pinMode(BUTTON, INPUT);
  pinMode(buttonTwo, INPUT);
  pinMode(buttonThree, INPUT);
  pinMode(PIN_OUT, OUTPUT);
  digitalWrite(PIN_OUT, LOW);
  pinMode(12, OUTPUT);
  digitalWrite(12, LOW);
  delay(200);
  // connect to Wi-Fi
  WiFiMulti.addAP(ssid.c_str(), password.c_str());
  while (WiFiMulti.run() != WL_CONNECTED)
  {
    Serial.print(".");
    delay(100);
  }
  Serial.println();
  // begin websocket
  webSocket.begin();
  webSocket.onEvent(webSocketEvent);
  // server address and port
  Serial.print("Please put the following IP address in the client sketch: "); Serial.print(WiFi.localIP());
  Serial.print(", port: "); Serial.println(WS_PORT);
}

void loop(){
  // need to be in this mode to receive a message
  while(digitalRead(buttonThree) != HIGH)
  {
    webSocket.loop();
  }
  // press button two after inputting a character to send message
  while (digitalRead(buttonTwo) != HIGH)
  {
    String myMsg = GetLetter();
    Translate(myMsg);
  }
  // send text across web socket
  webSocket.sendTXT(nums, finalMsg);
  finalMsg = "";
  Serial.println();
}

// outputs string to LEDs in morse code
void outputToLED(String message)
{
  String morseWord = encode(message);
  for(int i = 0; i <= morseWord.length(); i++)
  {
    switch(morseWord[i])
    {
      case '.': // dit
        digitalWrite(PIN_OUT, HIGH);
        delay(UNIT_LENGTH);
        digitalWrite(PIN_OUT, LOW);
        delay(UNIT_LENGTH);
        break;

      case '-': // dah
        digitalWrite(PIN_OUT, HIGH);
        delay(UNIT_LENGTH * 3);
        digitalWrite(PIN_OUT, LOW);
        delay(UNIT_LENGTH);
        break;

      case ' ': // gap
        delay(UNIT_LENGTH);
    }
  }
  digitalWrite(12, HIGH);
  delay(UNIT_LENGTH);
  digitalWrite(12, LOW);
  delay(UNIT_LENGTH * 3);
}

// reads input from button to morse code
String GetLetter()
{
  unsigned long START, STOP, TIME;
  String myMsg = "";
  while(true)
  {
    while(digitalRead(BUTTON) == LOW)
    {
      // do nothing until button is pressed
    }
    digitalWrite(PIN_OUT, HIGH);
    START = millis();
    while(digitalRead(BUTTON) == HIGH)
    {
      // do not register input until button is released
    }
    STOP = millis();
    digitalWrite(PIN_OUT, LOW);
    TIME = STOP - START;
    if (TIME > 25)
    {
      myMsg += Dot_or_Dash(TIME);
    }
    while((millis() - STOP) < 750)
    {
      delay(10);
      if (digitalRead(BUTTON) == HIGH)
      {
        break;
      }
    }
    if ((millis() - STOP) >= 750)
    {
      return myMsg;
    }
  }
}

// determine if it is a dot or dash
char Dot_or_Dash (unsigned long TIME)
{
  char DoD;
  if (TIME < 500)
  {
    DoD = '.';
  }
  else if (TIME >= 500 && TIME < 5000)
  {
    DoD = '-';
  }
  else
  {
    DoD = ' ';
  }
  return DoD;
}

// translates morse code to letters
void Translate (String myMsg)
{
  for (int i = 0; i < 47; i++)
  {
    if (myMsg == dots_and_dashes[i])
    {
      Serial.print(codex[i]);
      finalMsg += codex[i];
      return ;
    }
  }
  Serial.print("ERROR, I cannot understand you!\n");
  return ;
}

// translates letters to morse code
String encode(String string)
{
  size_t i, j;
  String morseWord = "";
  for( i = 0; string[i]; i++ )
  {
    for( j = 0; j < 47; j++ )
    {
      if( string[i] == codex[j] )
      {
        morseWord += dots_and_dashes[j];
        break;
      }
    }
    morseWord += " "; // Add tailingspace to separate the chars
  }
  return morseWord;
}
