/* Developed for Lenovo Educational Board Pass 2
   © Lenovo Created by Joshua Grosser
   v. 1.0 */

#include <Wire.h>   // I2C library
#include "ens210.h" // ENS210 library
#include <SparkFun_APDS9960.h> // ambient light library

// create objects
SparkFun_APDS9960 apds = SparkFun_APDS9960();
ENS210 ens210;

uint16_t ambient_light = 0; // ambient light variable

void setup() {
  // Enable serial
  Serial.begin(9600);

  // Enable I2C
  Wire.begin();

  // Enable ENS210 and APDS
  ens210.begin();
  apds.init();
  apds.enableLightSensor(false);
}

void loop() {
  // creates variables for temperature and humidity and measures new values
  int t_data, t_status, h_data, h_status;
  ens210.measure(&t_data, &t_status, &h_data, &h_status); 
  Serial.print("Temp: ");
  Serial.print( ens210.toFahrenheit(t_data, 10) / 10.0 );
  Serial.println(" F");
  Serial.print("Humidity: ");
  Serial.print( ens210.toPercentageH(h_data,1)); Serial.println(" %RH");
  // reads ambient light value
  apds.readAmbientLight(ambient_light);
  Serial.print("Ambient Light: "); Serial.print(ambient_light);
  Serial.println();
  Serial.println();

  // prints weather condition predictions based on humidity and ambient light
  if(ambient_light < 12000 && ens210.toPercentageH(h_data,1) > 75)
  {
    Serial.println("It is probably raining");
  }
  else if(ambient_light < 20000)
  {
    Serial.println("It is cloudy");
  }
  else if(ambient_light > 50000)
  {
    Serial.println("It is sunny");
  }
  else
  {
    Serial.println("There is a mix of sun and clouds");
  }
  Serial.println();
  // waits 2 seconds
  delay(2000);
}
